require 'digest'
require 'libarchive_rs'

# TODO we could also do gpg key verification here, but apt usually has
# per-package gpg verification turned off by default anyway.
#
# Something to keep in mind for the future.
module PackagecloudDebtools
  class Deb
    attr_accessor :fields, :io, :control_file
    def initialize(io, fields)
      @fields = fields
      @io = io

      extract_internal_archives
      compute_digests
      compute_files
    end

    def add_to_current_section(line, section)
      @fields[section] << line
    end

    # this function needs to be called after parsing has completed so we can
    # ensure we are calculating the md5 of the description after the entire
    # description has been read (descriptions can be multiline fields).
    def fix_description_and_compute_hash
      # as per the deb spec, description hashes include the trailing newline
      # see also: https://wiki.debian.org/RepositoryFormat#Description-md5
      desc_string = @fields['description'].join
      @fields['description'] = desc_string
      @fields['description_md5'] = Digest::MD5.hexdigest(desc_string)
    end

    private
    def compute_digests
      digest = Digester.from_io(@io)
      @fields.merge!('md5sum'    =>      digest.md5,
                     'sha1sum'   =>      digest.sha1,
                     'sha256sum' =>      digest.sha256,
                     'sha512sum' =>      digest.sha512)
    end

    def compute_files
      @fields['files'] = Array.new
      while entry = @data_archive.next_header
        # skip the / directory which will probably be present in every
        # deb.
        next if entry.pathname == './'
        digest = Digester.from_archive(@data_archive)
        @fields['files'] <<  { path: entry.pathname[1..-1],
                               mode:  entry.mode,
                               owner: entry.uname,
                               uid:  entry.uid,
                               group: entry.gname,
                               gid: entry.gid,
                               mtime: Time.at(Integer(entry.mtime)),
                               size: entry.size,
                               rdev_major: entry.rdevmajor,
                               rdev_minor: entry.rdevminor,
                               symlink: entry.symlink,
                               md5sum: digest.md5,
                               sha1sum: digest.sha1,
                               sha256sum: digest.sha256,
                               sha512sum: digest.sha512 }
      end
    end

    def files
      @fields['files']
    end

    def extract_internal_archives
      # deb files are ar archives, so we need to actually open the
      # archives
      #
      # if the deb archive was gpg signed, we could verify it here, too
      ar_archive = ::Archive.read_open_filename(@io.path) 
      while entry = ar_archive.next_header
        if entry.pathname =~ /control\.tar\.?.*/
          control_archive = ::Archive.read_open_memory(ar_archive.read_data)
          while control_entry = control_archive.next_header
            if control_entry.pathname == "./control"
              @control_file = control_archive.read_data.force_encoding('utf-8')
            end
          end
        elsif entry.pathname =~ /data\.tar\.?.*/
          @data_archive = ::Archive.read_open_memory(ar_archive.read_data)
        end
      end
    end
  end
end
